-- Copyright
-- Absorber
-- Author: Steven Bartels
-- Date: 07.07.2015

-- Addon Ref
local Absorber = LibStub("AceAddon-3.0"):NewAddon("Absorber", "AceConsole-3.0","AceEvent-3.0")

-- Globals
local SpellID = 0
local TotalShield = 0
local BuffAbsorb = 0

local MaxShield = 25000
local Locked = true
local IsVisible = true
local xPos = 200
local yPos = 100
local Width = 100
local Height = 25
local TextScale = 1
local TextPosX = 0
local TextPosY = 0
local TextColor = {r = 1,g = 1,b = 1}                   -- rgb
local BackgroundColor = {r = 1,g = 1,b = 1,a = 1}       -- rgba
local ForgroundColor = {r = 1,g = 1,b = 1,a = 1}        -- rgba

local Buffs = {
    Guard = {"Guard", 115295, false},                       -- Guard
    LifeCocoon = {"Life Cocoon", 116849, false},            -- Life Cocoon
    SacredShield = {"Sacred Shield",65148,false},           -- Sacred Shield
    AvengerReprieve = {"Avenger's Reprieve", 185676, false},-- Avenger's Reprieve
    DivineAegis = {"Divine Aegis", 47753, false},           -- Divine Aegis
    PowerwordShield = {"Powerword:Shield", 17, false},      -- Powerword:Shield
    SpiritShell = {"Spirit Shell", 114908, false},          -- Spirit Shell
    ShieldBarrier = {"Shield Barrier", 112048, false},      -- Shield Barrier
    IceBarrier = {"Ice Barrier", 11426, false},             -- Ice Barrier
    BloodShield = {"Blood Shield", 77535, false},           -- Blood Shield
    AntiMagicShell = {"Anti-Magic Shell", 48707, false},    -- Anit-Magic Shell    
}

local CustomBuffs = {}

local defaults = {
  profile = {
    MaxShield = 25000,
    Locked = true,
    IsVisible = true,
    x = 200,
    y = 100,
    Width = 100,
    Height = 25,
    TextScale = 1,
    TextPosX = 0,
    TextPosY = 0,
    TextColor = {r = 1,g = 1,b = 1},  
    BackgroundColor = {r = 1,g = 1,b = 1,a = 1},
    ForgroundColor = {r = 1,g = 1,b = 1,a = 1},  
    Buffs = {
        Guard = {"Guard", 115295, false},                       -- Guard
        LifeCocoon = {"Life Cocoon", 116849, false},            -- Life Cocoon
        SacredShield = {"Sacred Shield",65148,false},           -- Sacred Shield
        AvengerReprieve = {"Avenger's Reprieve", 185676, false},-- Avenger's Reprieve
        DivineAegis = {"Divine Aegis", 47753, false},           -- Divine Aegis
        PowerwordShield = {"Powerword:Shield", 17, false},      -- Powerword:Shield
        SpiritShell = {"Spirit Shell", 114908, false},          -- Spirit Shell
        ShieldBarrier = {"Shield Barrier", 112048, false},      -- Shield Barrier
        IceBarrier = {"Ice Barrier", 11426, false},             -- Ice Barrier
        BloodShield = {"Blood Shield", 77535, false},           -- Blood Shield
        AntiMagicShell = {"Anti-Magic Shell", 48707, false},    -- Anit-Magic Shell
    },
    CustomBuffs = {}
  }
}

-- Ace OnInitialize
function Absorber:OnInitialize()
    DataBase = LibStub("AceDB-3.0"):New("Absorber", defaults)
end

-- Ace OnEnable
function Absorber:OnEnable()
    Absorber:CheckDataBase()
    AbsorberUI:CreateAbsorbBar()
    Absorber:RegisterEvent("UNIT_AURA", "PerformAuraCheck")
    Absorber:RegisterChatCommand("absorber", "SlashCommandFunc")

end

-- Ace OnDisable
function Absorber:OnDisable()
    Absorber:UnregisterEvent("UNIT_AURA")
    Absorber:UnregisterChatCommand("absorber")
end

-- Catching /Commands
function Absorber:SlashCommandFunc(input)
    if input == "help" then
        Absorber:ShowHelp()
    elseif input == "lock" then
        SetLocked(true)        
        Absorber:Print("Absorber is locked now!") 
    elseif input == "unlock" then
        SetLocked(false)    
        Absorber:Print("Absorber is unlocked now!")
    elseif input == "show" and IsVisible == false then
        SetVisibility(true)   
        Absorber:Print("Absorber is visible now!")
    elseif input == "hide" and IsVisible == true then
        SetVisibility(false)    
        Absorber:Print("Absorber is hidden now!")
    elseif input == "config" then
        AbsorberUI:OpenOptions()
    elseif input == "reset" then
        Absorber:ResetValues()
    elseif input == "remove" then
        Absorber:RemoveAllCustomBuffs()
    end
end

-- Performs an update of the UI
function Absorber:PerformUpdate()
    UpdateBar()
end

-- Simple Calculation for total shield
function Absorber:CalculateShield(currentValue)
    if  (currentValue ~= nil ) then
		TotalShield = TotalShield + currentValue
	end
    Absorber:PerformUpdate()
end

-- Check of aura/buff changes
function Absorber:PerformAuraCheck(arg1, arg2)
    if arg2 == "player" then
       TotalShield = 0
       for i=1,25 do
            local SpellFound = false
			_, _, _, _, _, _, _, _, _, _, SpellID, _, _, _,BuffAbsorb = UnitBuff("Player", i)
            -- Check NIL
            if SpellID == nil then
                break
            end

            -- Check First Database
            for key,value in pairs(Buffs) do
                if SpellID == value[2] and value[3] then
                    SpellFound = true
                    break
                end
            end

            -- Check Second Database
            if not SpellFound then
                for key,value in pairs(CustomBuffs) do
                    if SpellID == tonumber(value[2]) and value[3] then
                        SpellFound = true
                        break
                    end
                end
            end

            -- Calculate Shield if Spell was found
            if SpellFound then 
                Absorber:CalculateShield(BuffAbsorb)         
            end
       end
       Absorber:PerformUpdate()
    end   
end

-- Help
function Absorber:ShowHelp()
     Absorber:Print("Absorber commands:")  
     Absorber:Print("/absorber   help - Showing the help")
     Absorber:Print("/absorber config - Opens the options")
     Absorber:Print("/absorber   lock - Locks the bar") 
     Absorber:Print("/absorber unlock - Unlocks the bar")  
     Absorber:Print("/absorber   show - Makes the bar visible")  
     Absorber:Print("/absorber   hide - Makes the bar invisible")
     Absorber:Print("/absorber  reset - Resets all value to default settings")
     Absorber:Print("/absorber remove - Removes ALL custom buffs!")
     
     Absorber:Print("by Kaldarus <Call for Sanity> on Kult der Verdammten")
end

-- Visibilty
function GetVisibility()
    return IsVisible
end

function SetVisibility(value)
    IsVisible = value
    Absorber:SaveDataBase()
    Absorber:PerformUpdate()
end

function Absorber:SaveDataBase()
    DataBase.char.Locked = Locked
    DataBase.char.IsVisible = IsVisible
    DataBase.char.MaxShield = MaxShield

    DataBase.char.x = xPos
    DataBase.char.y = yPos
    DataBase.char.Height = Height
    DataBase.char.Width = Width

    DataBase.char.TextScale = TextScale
    DataBase.char.TextPosY = TextPosY
    DataBase.char.TextPosX = TextPosX
    DataBase.char.TextColor = TextColor

    DataBase.char.Buffs = Buffs
    DataBase.char.CustomBuffs = CustomBuffs
    --Absorber:Print("Save complete!")
end

function Absorber:LoadDataBase()
    Locked = DataBase.char.Locked
    IsVisible = DataBase.char.IsVisible
    MaxShield = DataBase.char.MaxShield

    xPos = DataBase.char.x 
    yPos = DataBase.char.y
    Height = DataBase.char.Height
    Width = DataBase.char.Width
    BackgroundColor = DataBase.char.BackgroundColor
    ForgroundColor = DataBase.char.ForgroundColor

    TextScale = DataBase.char.TextScale
    TextPosY = DataBase.char.TextPosY
    TextPosX = DataBase.char.TextPosX
    TextColor = DataBase.char.TextColor

    Buffs = DataBase.char.Buffs
    CustomBuffs = DataBase.char.CustomBuffs
    --Absorber:Print("Load complete!")
end

function Absorber:CheckDataBase()
    for Key, Value in pairs(defaults.profile) do
        if DataBase.char[Key] == nil then
            DataBase.char[Key] = Value
        end
    end
    for Key, Value in pairs(defaults.profile.Buffs) do
        if DataBase.char.Buffs[Key] == nil then
            DataBase.char.Buffs[Key] = Value
        end
    end         
    Absorber:LoadDataBase()
end

function SetPos(xValue, yValue)
    xPos = xValue
    yPos = yValue
    Absorber:SaveDataBase()
end

function GetPos()
    return xPos, yPos
end

function GetTotalShield()
    return TotalShield
end

function GetMaxShield()
    return MaxShield
end

function SetMaxShield(value)
    MaxShield = value
    Absorber:SaveDataBase()
end

function GetHeight()
    return Height
end

function GetWidth()
    return Width
end


function SetHeight(value)
    Height = value
    Absorber:SaveDataBase()
    SetBarHeight(Height)
end

function SetWidth(value)
    Width = value
    Absorber:SaveDataBase()
    SetBarWidth(Width)
end

-- Lock/Unlock
function GetLocked()
    return Locked
end

function SetLocked(value)
    Locked = value
    Absorber:SaveDataBase()
end

function SetBackgroundColor(_,_,value_r, value_g, value_b, value_a)
    BackgroundColor.r = value_r
    BackgroundColor.g = value_g
    BackgroundColor.b = value_b
    BackgroundColor.a = value_a
    Absorber:SaveDataBase()
    SetBarBackgroundColor(_,_,BackgroundColor.r,BackgroundColor.g ,BackgroundColor.b, BackgroundColor.a)
end

function GetBackgroundColor()
    return BackgroundColor.r, BackgroundColor.g, BackgroundColor.b, BackgroundColor.a
end

function SetForgroundColor(_,_,value_r, value_g, value_b, value_a)
    ForgroundColor.r = value_r
    ForgroundColor.g = value_g
    ForgroundColor.b = value_b
    ForgroundColor.a = value_a
    Absorber:SaveDataBase()
    SetBarForgroundColor(_,_,ForgroundColor.r,ForgroundColor.g ,ForgroundColor.b, ForgroundColor.a)
end

function GetForgroundColor()
    return ForgroundColor.r, ForgroundColor.g, ForgroundColor.b, ForgroundColor.a
end

function SetTextColor(_,_,value_r, value_g, value_b)
    TextColor.r = value_r
    TextColor.g = value_g
    TextColor.b = value_b
    Absorber:SaveDataBase()
    SetBarTextColor(_,_,TextColor.r,TextColor.g ,TextColor.b)
end

function GetTextColor()
    return TextColor.r, TextColor.g, TextColor.b
end

function SetTextScale(value)
    TextScale = value;
    Absorber:SaveDataBase()
    SetBarTextScale(TextScale)
end

function GetTextScale()
    return TextScale
end

function SetTextPosX(value)
    TextPosX = value;
    Absorber:SaveDataBase()
    SetBarTextPositionX(TextPosX)
end

function SetTextPosY(value)
    TextPosY = value;
    Absorber:SaveDataBase()
    SetBarTextPositionY(TextPosY)
end

function GetTextPos()
    return TextPosX, TextPosY
end

function AddAllCustomBuffs()
    for KeyTmp, ValueTmp in pairs(CustomBuffs) do
        AddCustomBuffCheckBox(KeyTmp)
    end  
end

function SetEnableBuff(key, value)
    local spellfound = false
    for KeyTmp, ValueTmp in pairs(Buffs) do
        if KeyTmp == key then
           ValueTmp[3] = value
           spellfound = true
        end
    end        
    if not spellfound then
        for KeyTmp, ValueTmp in pairs(CustomBuffs) do
            if KeyTmp == key then
               ValueTmp[3] = value
            end
        end  
    end
    Absorber:SaveDataBase()
end

function GetEnableBuff(key)
    local value = nil
    for KeyTmp, ValueTmp in pairs(Buffs) do
        if KeyTmp == key then
           value = ValueTmp[3]
        end
    end        
    if value == nil then
        for KeyTmp, ValueTmp in pairs(CustomBuffs) do
            if KeyTmp == key then
               value = ValueTmp[3]
            end
        end  
    end
    return value 
end

function GetBuffName(key)
    local value = nil
    for KeyTmp, ValueTmp in pairs(Buffs) do
        if KeyTmp == key then
           value = ValueTmp[1]
        end
    end        
    if value == nil then
        for KeyTmp, ValueTmp in pairs(CustomBuffs) do
            if KeyTmp == key then
               value = ValueTmp[1]
            end
        end  
    end
    return value
end

function GetBuffIcon(key)
    local value = nil
    local icon
    for KeyTmp, ValueTmp in pairs(Buffs) do
        if KeyTmp == key then
           value = ValueTmp[2]
        end
    end        
    if value == nil then
        for KeyTmp, ValueTmp in pairs(CustomBuffs) do
            if KeyTmp == key then
               value = ValueTmp[2]
            end
        end  
    end
    if value ~= nil then
        _, _, icon, _, _, _ = GetSpellInfo(value)
    end
    return icon
end

function AddCustomBuff(name, id)
    local size = 0
    if Buffs ~= nil then
        for key, value in pairs(Buffs) do
            if value[2] == id then
                Absorber:Print("Could not add Buff! Buff with ID ".. id .. " already in Database!")
                return 
            end
        end
    end
    if CustomBuffs ~= nil then
        for key, value in pairs(CustomBuffs) do
            size = size + 1
            if value[2] == id then
                Absorber:Print("Could not add Buff! Buff with ID ".. id .. " already in Database!")
                return 
            end
        end
    end
    local ElementName = "CustomSpell" .. size
    CustomBuffs[ElementName] = {name, id, false}

    Absorber:Print("Buff with ID ".. id .. " added to Database!")
    AddCustomBuffCheckBox(ElementName)
end

function RemoveCustomBuff(id)
    for key, value in pairs(CustomBuffs) do
        if value[2] == id then
            key = nil
            Absorber:Print("Buff with ID " .. id .. " was removed from the Database!")
            Absorber:SaveDataBase()
            return 
        end
    end
    Absorber:Print("Buff with ID" .. id .. "not found in Database!")
end

function Absorber:RemoveAllCustomBuffs()
    CustomBuffs = {}
    Absorber:SaveDataBase()
end

function Absorber:ResetValues()
    Locked = defaults.profile.Locked
    IsVisible = defaults.profile.IsVisible
    MaxShield = defaults.profile.MaxShield

    xPos = defaults.profile.x 
    yPos = defaults.profile.y
    Height = defaults.profile.Height
    Width = defaults.profile.Width
    BackgroundColor = defaults.profile.BackgroundColor
    ForgroundColor = defaults.profile.ForgroundColor
    
    TextScale = defaults.profile.TextScale
    TextPosX = defaults.profile.TextScale
    TextPosY = defaults.profile.TextScale
    TextColor = defaults.profile.TextColor

    Buffs = defaults.profile.Buffs
    CustomBuffs = {}

    ResetBar()
    Absorber:SaveDataBase()
    Absorber:PerformUpdate()  
    Absorber:Print("Reset complete!")
end