-- Copyright
-- Absorber
-- Author: Steven Bartels
-- Date: 07.07.2015    

AbsorberUI = LibStub("AceGUI-3.0")
local Media = LibStub("LibSharedMedia-3.0")
Media:Register("background", "Absorb_DefaultTexture", "Interface\\Addons\\Absorber\\Images\\Default_Texture.tga")
Media:Register("background", "Absorb_TB_Radient", "Interface\\Addons\\Absorber\\Images\\TopBottomRadiant.tga")
Media:Register("background", "Absorb_BT_Radient", "Interface\\Addons\\Absorber\\Images\\BottomTopRadiant.tga")

-- Options
local OptionsOpen = false

-- Bar
local Options = AbsorberUI:Create("Frame") 
Options:Hide()  
local BarContainer = AbsorberUI:Create("SimpleGroup")
local BarForground = AbsorberUI:Create("Label")
local BarBackground = AbsorberUI:Create("Label")
local ValueLabel = AbsorberUI:Create("Label")
local ClassScroll = AbsorberUI:Create("ScrollFrame")

function AbsorberUI:OpenOptions()
    if not OptionsOpen then     
        Options:SetTitle("Absorber Options")
        Options:SetStatusText("by Kaldarus <Call for Sanity> - Kult der Verdammten")
        Options:SetLayout("Flow")    
        Options:SetCallback("OnClose", function(widget) AbsorberUI:Release(widget) OptionsOpen = false Options:Hide() end)
        Options:SetPoint("CENTER",0,0)
        Options:EnableResize(false)
        Options:SetHeight(400)
        Options:SetWidth(830)
        Options:Show()

        AbsorberUI:CreateBarSetting(Options, 330, 250)
        AbsorberUI:CreateTextSettings(Options, 330, 250)
        AbsorberUI:CreateBuffSettings(Options, 330, 280)
        
    end
end

function AbsorberUI:CreateAbsorbBar()
    BarContainer:SetLayout("FULL")
    BarContainer.frame:SetPoint("CENTER",DataBase.char.x, DataBase.char.y)
    BarContainer.frame:SetSize(GetWidth(),GetHeight())
    BarContainer.frame:SetMovable(true)

    local r,g,b,a = GetBackgroundColor()

    BarBackground:SetImage(Media:Fetch("background", "Absorb_DefaultTexture"))
    BarBackground.image:SetVertexColor(r,g,b,a)
    BarBackground:SetImageSize(GetWidth(),GetHeight())
    BarBackground.frame:SetMovable(true)   
    BarBackground.frame:SetFrameStrata("BACKGROUND") 
    BarBackground.frame:SetScript("OnMouseDown",BarOnMouseDown)
    BarBackground.frame:SetScript("OnMouseUp",BarOnMouseUp)
    BarBackground:SetPoint("LEFT",BarContainer.frame,0, 0)
    BarContainer:AddChild(BarBackground)

    r,g,b,a = GetForgroundColor()

    BarForground:SetImage(Media:Fetch("background", "Absorb_DefaultTexture"))
    BarForground.image:SetVertexColor(r,g,b,a)
    BarForground.image:SetBlendMode("ADD")
    BarForground:SetImageSize(0,GetHeight())
    BarForground.frame:SetMovable(true)
    BarForground.frame:SetFrameStrata("MEDIUM")
    BarForground.frame:SetPoint("LEFT",BarContainer.frame, 0, 0)
    BarContainer:AddChild(BarForground)

    
    local TextX, TextY = GetTextPos()
    local r,g,b = GetTextColor()

    ValueLabel:SetText(GetTotalShield())
    ValueLabel:SetColor(r,g,b)
    ValueLabel.frame:SetScale(GetTextScale())
    ValueLabel.frame:SetMovable(true)
    ValueLabel.frame:SetFrameStrata("High")
    ValueLabel.label:SetJustifyH("CENTER")
	ValueLabel.label:SetJustifyV("CENTER")
    ValueLabel.frame:SetPoint("CENTER", TextX, TextY)

    BarContainer:AddChild(ValueLabel)

    UpdateBar()
end

function AbsorberUI:CreateBuffSettings(parent, height, width)
        local CustomBuffGroup = AbsorberUI:Create("InlineGroup")
        CustomBuffGroup:SetTitle("Buff Settings")
        CustomBuffGroup:SetAutoAdjustHeight(false)
        CustomBuffGroup:SetLayout("Flow")
        CustomBuffGroup:SetHeight(height)
        CustomBuffGroup:SetWidth(width)
        parent:AddChild(CustomBuffGroup)
        
        -- Buff ID Textbox
        local BuffIDBox = AbsorberUI:Create("EditBox")
        BuffIDBox:SetLabel("Spell ID")
        BuffIDBox:SetText("")
        BuffIDBox:SetWidth(80)
        BuffIDBox:DisableButton(true)
        BuffIDBox.editbox:SetNumeric()
        CustomBuffGroup:AddChild(BuffIDBox)

        -- Buff ID Textbox
        local SaveButton = AbsorberUI:Create("Button")
        SaveButton:SetText("Add Custom Spell")
        SaveButton:SetWidth(150)
        SaveButton:SetHeight(25)
        SaveButton:SetCallback("OnClick", function(widget) local name, _, _, _, _, _ = GetSpellInfo(BuffIDBox:GetText())
                                                                 AddCustomBuff(name, BuffIDBox:GetText())
                                                                 BuffIDBox:SetText("") 
                                                                 AbsorberUI:ClearFocus() 
                          end)
        CustomBuffGroup:AddChild(SaveButton)

        AbsorberUI:CreateClassSettings(CustomBuffGroup, 220, width)
end

function AbsorberUI:CreateClassSettings(parent, height, width)
        local ClassGroup = AbsorberUI:Create("SimpleGroup")
        ClassGroup:SetAutoAdjustHeight(false)
        ClassGroup:SetLayout("Fill")
        ClassGroup:SetHeight(height)
        ClassGroup:SetWidth(width)
        parent:AddChild(ClassGroup)

        ClassScroll:SetLayout("Flow")
        ClassScroll:SetHeight(330)
        ClassScroll:SetWidth(280)
        ClassGroup:AddChild(ClassScroll)
     
        AddCheckBox(ClassScroll,"SacredShield")
        AddCheckBox(ClassScroll,"AvengerReprieve")
        AddCheckBox(ClassScroll,"Guard")
        AddCheckBox(ClassScroll,"LifeCocoon")
        AddCheckBox(ClassScroll,"PowerwordShield")
        AddCheckBox(ClassScroll,"DivineAegis")
        AddCheckBox(ClassScroll,"SpiritShell")
        AddCheckBox(ClassScroll,"ShieldBarrier")
        AddCheckBox(ClassScroll,"IceBarrier")
        AddCheckBox(ClassScroll,"BloodShield")
        AddCheckBox(ClassScroll,"AntiMagicShell")

        AddAllCustomBuffs()
end

function AbsorberUI:CreateBarSetting(parent, height, width)
        local BarGroup = AbsorberUI:Create("InlineGroup")
        BarGroup:SetTitle("Bar Settings")
        BarGroup:SetAutoAdjustHeight(false)
        BarGroup:SetLayout("List")
        BarGroup:SetHeight(height)
        BarGroup:SetWidth(width)
        parent:AddChild(BarGroup)

        -- Absorb Textbox
        local Editbox = AbsorberUI:Create("EditBox")
        Editbox:SetLabel("Max Aborb Amount Shown")
        Editbox:SetText(GetMaxShield())
        Editbox:SetWidth(200)
        Editbox:DisableButton(true)
        Editbox:SetCallback("OnEnterPressed", function(widget) SetMaxShield(Editbox.editbox:GetNumber()) AbsorberUI:ClearFocus() end)
        Editbox.editbox:SetNumeric()
        BarGroup:AddChild(Editbox)

        -- Width and Hight Slider
        local WidthSlider = AbsorberUI:Create("Slider")
        WidthSlider:SetLabel("Bar Width")
        WidthSlider:SetValue(GetWidth())
        WidthSlider:SetSliderValues(10,500,5)
        WidthSlider:SetWidth(200)
        WidthSlider:SetCallback("OnValueChanged", function(widget) SetBarWidth(WidthSlider.slider:GetValue()) end)
        WidthSlider:SetCallback("OnMouseUp", function(widget) SetWidth(WidthSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        BarGroup:AddChild(WidthSlider)

        local HeightSlider = AbsorberUI:Create("Slider")
        HeightSlider:SetLabel("Bar Height")
        HeightSlider:SetValue(GetHeight())
        HeightSlider:SetSliderValues(10,500,5)
        HeightSlider:SetWidth(200)
        HeightSlider:SetCallback("OnValueChanged", function(widget) SetBarHeight(HeightSlider.slider:GetValue()) end)
        HeightSlider:SetCallback("OnMouseUp", function(widget) SetHeight(HeightSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        BarGroup:AddChild(HeightSlider)
        
        local r,g,b,a = GetForgroundColor()

        local ForgroundBarColorPicker = AbsorberUI:Create("ColorPicker")
        ForgroundBarColorPicker:SetLabel("Forground Bar Color")
        ForgroundBarColorPicker:SetColor(r,g,b,a)
        ForgroundBarColorPicker:SetHasAlpha(true)
        ForgroundBarColorPicker:SetCallback("OnValueChanged", SetBarForgroundColor)
        ForgroundBarColorPicker:SetCallback("OnValueConfirmed", SetForgroundColor)
        BarGroup:AddChild(ForgroundBarColorPicker)

        r,g,b,a = GetBackgroundColor()

        local BackgroundBarColorPicker = AbsorberUI:Create("ColorPicker")
        BackgroundBarColorPicker:SetLabel("Background Bar Color")
        BackgroundBarColorPicker:SetColor(r,g,b,a)
        BackgroundBarColorPicker:SetHasAlpha(true)
        BackgroundBarColorPicker:SetCallback("OnValueChanged", SetBarBackgroundColor)
        BackgroundBarColorPicker:SetCallback("OnValueConfirmed", SetBackgroundColor)
        BarGroup:AddChild(BackgroundBarColorPicker)

         -- Unlock/Lock CheckBox
        local LockCheckbox = AbsorberUI:Create("CheckBox")
        LockCheckbox:SetLabel("Unlock/Lock")
        LockCheckbox:SetValue(true)
        LockCheckbox:SetCallback("OnValueChanged", function(widget) SetLocked(LockCheckbox:GetValue()) end)
        BarGroup:AddChild(LockCheckbox)
end

function AbsorberUI:CreateTextSettings(parent, height, width)
        local TextGroup = AbsorberUI:Create("InlineGroup")
        TextGroup:SetTitle("Text Settings")
        TextGroup:SetLayout("List")
        TextGroup:SetAutoAdjustHeight(false)
        TextGroup:SetHeight(height)
        TextGroup:SetWidth(width)
        parent:AddChild(TextGroup)

        -- Text Size Slider
        local TextScaleSlider = AbsorberUI:Create("Slider")
        TextScaleSlider:SetLabel("Text Size")
        TextScaleSlider:SetValue(GetTextScale())
        TextScaleSlider:SetSliderValues(0.1,5,0.1)
        TextScaleSlider:SetWidth(200)
        TextScaleSlider:SetCallback("OnValueChanged", function(widget) SetBarTextScale(TextScaleSlider.slider:GetValue()) end)
        TextScaleSlider:SetCallback("OnMouseUp", function(widget) SetTextScale(TextScaleSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        TextGroup:AddChild(TextScaleSlider)

        local TextX, TextY = GetTextPos()

        -- Text X Slider
        local TextPosXSlider = AbsorberUI:Create("Slider")
        TextPosXSlider:SetLabel("Relative Text X Position")
        TextPosXSlider:SetValue(TextX)
        TextPosXSlider:SetSliderValues(-100,100,1)
        TextPosXSlider:SetWidth(200)
        TextPosXSlider:SetCallback("OnValueChanged", function(widget) SetBarTextPositionX(TextPosXSlider.slider:GetValue()) end)
        TextPosXSlider:SetCallback("OnMouseUp", function(widget) SetTextPosX(TextPosXSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        TextGroup:AddChild(TextPosXSlider)

        -- Text Y Slider
        local TextPosYSlider = AbsorberUI:Create("Slider")
        TextPosYSlider:SetLabel("Relative Text Y Position")
        TextPosYSlider:SetValue(TextY)
        TextPosYSlider:SetSliderValues(-100,100,1)
        TextPosYSlider:SetWidth(200)
        TextPosYSlider:SetCallback("OnValueChanged", function(widget) SetBarTextPositionY(TextPosYSlider.slider:GetValue()) end)
        TextPosYSlider:SetCallback("OnMouseUp", function(widget) SetTextPosY(TextPosYSlider.slider:GetValue()) AbsorberUI:ClearFocus() end)
        TextGroup:AddChild(TextPosYSlider)

        local r,g,b = GetTextColor()

        local TextColorPicker = AbsorberUI:Create("ColorPicker")
        TextColorPicker:SetLabel("Text Color")
        TextColorPicker:SetColor(r,g,b)
        TextColorPicker:SetHasAlpha(false)
        TextColorPicker:SetCallback("OnValueChanged", SetBarTextColor)
        TextColorPicker:SetCallback("OnValueConfirmed", SetTextColor)
        TextGroup:AddChild(TextColorPicker)
end

function AddCheckBox(parent, key)
        local Checkbox = AbsorberUI:Create("CheckBox")
        Checkbox:SetLabel(GetBuffName(key))
        Checkbox:SetValue(GetEnableBuff(key))
        Checkbox:SetWidth(100)
        Checkbox.frame:SetWidth(200) 
        Checkbox:SetCallback("OnValueChanged", function(widget) SetEnableBuff(key,Checkbox:GetValue()) end)
        parent:AddChild(Checkbox)

        local Icon = AbsorberUI:Create("Icon")
        Icon:SetImage(GetBuffIcon(key))
        Icon:SetImageSize(20,20)
        Icon.frame:SetSize(20,20)
        Icon.frame:Disable(true)
        parent:AddChild(Icon)
end

function AddCustomBuffCheckBox(key)
    AddCheckBox(ClassScroll,key)
end

-- Events
function BarOnMouseDown(_, button)
    if button == "LeftButton" and not GetLocked() then
		BarContainer.frame:StartMoving()  
    end
end

function BarOnMouseUp(_, button)
    local xPos = 0;
    local yPos = 0;
    if button == "LeftButton" and not GetLocked() then
		BarContainer.frame:StopMovingOrSizing()    
        _, _, _, xPos, yPos = BarContainer:GetPoint()
    SetPos(xPos,yPos)
    end
end

function UpdateBar()
    local xPos, yPos
    local barvalue = (GetTotalShield() / GetMaxShield())
    local barwidth = barvalue * GetWidth()
    local TextX, TextY = GetTextPos()

    if barwidth <= 1 then
        BarForground.frame:Hide()
    else 
        _,_,_,xPos,yPos = BarBackground:GetPoint()
        BarForground.frame:Show()
        BarForground:SetImageSize(barwidth,GetHeight())
        BarForground.frame:SetPoint("LEFT", xPos -(GetWidth() - barwidth)/2,yPos)
    end
    ValueLabel.frame:SetPoint("CENTER", TextX, TextY)
    ValueLabel:SetText(GetTotalShield())
    SetBarVisibility()
end

function SetBarVisibility()
    if DataBase.char.IsVisible then
        BarContainer.frame:Show()
    elseif not DataBase.char.IsVisible then
        BarContainer.frame:Hide()
    end
end

function SetBarWidth(value)
    BarContainer.frame:SetSize(value,GetHeight())
    BarBackground:SetImageSize(value,GetHeight())
    UpdateBar()
end

function SetBarHeight(value)
    BarContainer.frame:SetSize(GetWidth(),value)
    BarBackground:SetImageSize(GetWidth(),value)
    UpdateBar()
end

function SetBarBackgroundColor(_,_,r,g,b,a)
    BarBackground.image:SetVertexColor(r,g,b,a)
end

function SetBarForgroundColor(_,_,r,g,b,a)
    BarForground.image:SetVertexColor(r,g,b,a)
end

function SetBarTextColor(_,_,r,g,b)
    ValueLabel:SetColor(r, g, b)
end

function SetBarTextScale(value)
    ValueLabel.frame:SetScale(value)
end

function SetBarTextPositionX(value)
    local _, TextY = GetTextPos()
    ValueLabel.frame:SetPoint("CENTER", value, TextY)
end


function SetBarTextPositionY(value)
    local TextX, _ = GetTextPos()
    ValueLabel.frame:SetPoint("CENTER", TextX, value)
end


function ResetBar()
    BarContainer.frame:SetSize(GetWidth(),GetHeight())
    BarBackground:SetImageSize(GetWidth(),GetHeight())
    BarForground:SetImageSize(0,GetHeight())
    
    BarContainer.frame:SetPoint("LEFT",DataBase.char.x, DataBase.char.y)
    BarBackground:SetPoint("LEFT",BarContainer.frame,0, 0)
    BarForground.frame:SetPoint("LEFT",BarContainer.frame, 0, 0)
    ValueLabel.frame:SetPoint("CENTER", 0, 0)
end